@@ -31,6 +31,7 @@
 #include "common/string-utils.h"
 #include "common/sysfs-utils.h"
 #include "common/messages.h"
+#include "common/tree-search.h"
 
 /*
  * Insert a root item for temporary tree root
@@ -47,6 +48,17 @@
 	.name ## _str = NULL,					\
 	.name ## _ver = 0
 
+/*
+ * For feature names that are only an alias we don't need to duplicate
+ * versions.
+ *
+ * When compat_str is NULL, the feature descriptor is an alias.
+ */
+#define VERSION_ALIAS						\
+		VERSION_NULL(compat),				\
+		VERSION_NULL(safe),				\
+		VERSION_NULL(default)
+
 enum feature_source {
 	FS_FEATURES,
 	RUNTIME_FEATURES,
@@ -90,6 +102,9 @@ struct btrfs_feature {
 	const char *desc;
 };
 
+/*
+ * Keep the list sorted by compat version.
+ */
 static const struct btrfs_feature mkfs_features[] = {
 	{
 		.name		= "mixed-bg",
@@ -107,16 +122,7 @@ static const struct btrfs_feature mkfs_features[] = {
 		VERSION_TO_STRING2(compat, 3,4),
 		VERSION_NULL(safe),
 		VERSION_NULL(default),
-		.desc		= "quota support (qgroups)"
-	},
-	{
-		.name		= "squota",
-		.incompat_flag	= BTRFS_FEATURE_INCOMPAT_SIMPLE_QUOTA,
-		.sysfs_name	= "squota",
-		VERSION_TO_STRING2(compat, 6,7),
-		VERSION_NULL(safe),
-		VERSION_NULL(default),
-		.desc		= "squota support (simple accounting qgroups)"
+		.desc		= "hierarchical quota group support (qgroups)"
 	},
 	{
 		.name		= "extref",
@@ -151,6 +157,14 @@ static const struct btrfs_feature mkfs_features[] = {
 		VERSION_TO_STRING2(default, 5,15),
 		.desc		= "no explicit hole extents for files"
 	},
+	{
+		.name		= "fst",
+		.compat_ro_flag	= BTRFS_FEATURE_COMPAT_RO_FREE_SPACE_TREE |
+				  BTRFS_FEATURE_COMPAT_RO_FREE_SPACE_TREE_VALID,
+		.sysfs_name = "free_space_tree",
+		VERSION_ALIAS,
+		.desc		= "free-space-tree alias"
+	},
 	{
 		.name		= "free-space-tree",
 		.compat_ro_flag	= BTRFS_FEATURE_COMPAT_RO_FREE_SPACE_TREE |
@@ -159,7 +173,7 @@ static const struct btrfs_feature mkfs_features[] = {
 		VERSION_TO_STRING2(compat, 4,5),
 		VERSION_TO_STRING2(safe, 4,9),
 		VERSION_TO_STRING2(default, 5,15),
-		.desc		= "free space tree (space_cache=v2)"
+		.desc		= "free space tree, improved space tracking (space_cache=v2)"
 	},
 	{
 		.name		= "raid1c34",
@@ -178,18 +192,9 @@ static const struct btrfs_feature mkfs_features[] = {
 		VERSION_TO_STRING2(compat, 5,12),
 		VERSION_NULL(safe),
 		VERSION_NULL(default),
-		.desc		= "support zoned devices"
+		.desc		= "support zoned (SMR/ZBC/ZNS) devices"
 	},
 #endif
-	{
-		.name		= "block-group-tree",
-		.compat_ro_flag	= BTRFS_FEATURE_COMPAT_RO_BLOCK_GROUP_TREE,
-		.sysfs_name	= "block_group_tree",
-		VERSION_TO_STRING2(compat, 6,1),
-		VERSION_NULL(safe),
-		VERSION_NULL(default),
-		.desc		= "block group tree to reduce mount time"
-	},
 #if EXPERIMENTAL
 	{
 		.name		= "extent-tree-v2",
@@ -199,16 +204,51 @@ static const struct btrfs_feature mkfs_features[] = {
 		VERSION_NULL(safe),
 		VERSION_NULL(default),
 		.desc		= "new extent tree format"
-	} , {
+	},
+#endif
+	{
+		.name		= "bgt",
+		.compat_ro_flag	= BTRFS_FEATURE_COMPAT_RO_BLOCK_GROUP_TREE,
+		.sysfs_name	= "block_group_tree",
+		VERSION_ALIAS,
+		.desc		= "block-group-tree alias"
+	},
+	{
+		.name		= "block-group-tree",
+		.compat_ro_flag	= BTRFS_FEATURE_COMPAT_RO_BLOCK_GROUP_TREE,
+		.sysfs_name	= "block_group_tree",
+		VERSION_TO_STRING2(compat, 6,1),
+		VERSION_NULL(safe),
+		VERSION_NULL(default),
+		.desc		= "block group tree, more efficient block group tracking to reduce mount time"
+	},
+#if EXPERIMENTAL
+	{
+		.name		= "rst",
+		.incompat_flag	= BTRFS_FEATURE_INCOMPAT_RAID_STRIPE_TREE,
+		.sysfs_name	= "raid_stripe_tree",
+		VERSION_ALIAS,
+		.desc		= "raid-stripe-tree alias"
+	},
+	{
 		.name		= "raid-stripe-tree",
 		.incompat_flag	= BTRFS_FEATURE_INCOMPAT_RAID_STRIPE_TREE,
-		.sysfs_name	= NULL,
+		.sysfs_name	= "raid_stripe_tree",
 		VERSION_TO_STRING2(compat, 6,7),
 		VERSION_NULL(safe),
 		VERSION_NULL(default),
-		.desc		= "raid stripe tree"
+		.desc		= "raid stripe tree, enhanced file extent tracking"
 	},
 #endif
+	{
+		.name		= "squota",
+		.incompat_flag	= BTRFS_FEATURE_INCOMPAT_SIMPLE_QUOTA,
+		.sysfs_name	= "simple_quota",
+		VERSION_TO_STRING2(compat, 6,7),
+		VERSION_NULL(safe),
+		VERSION_NULL(default),
+		.desc		= "squota support (simple accounting qgroups)"
+	},
 	/* Keep this one last */
 	{
 		.name		= "list-all",
@@ -252,6 +292,11 @@ static const struct btrfs_feature runtime_features[] = {
 	}
 };
 
+static bool feature_name_is_alias(const struct btrfs_feature *feature)
+{
+	return feature->compat_str == NULL;
+}
+
 /*
  * This is a sanity check to make sure BTRFS_FEATURE_STRING_BUF_SIZE is large
  * enough to contain all strings.
@@ -270,12 +315,21 @@ void btrfs_assert_feature_buf_size(void)
 	for (i = 0; i < ARRAY_SIZE(mkfs_features); i++)
 		/* The extra 2 bytes are for the ", " prefix. */
 		total_size += strlen(mkfs_features[i].name) + 2;
-	BUG_ON(BTRFS_FEATURE_STRING_BUF_SIZE < total_size);
+
+	if (BTRFS_FEATURE_STRING_BUF_SIZE < total_size) {
+		internal_error("string buffer for freature list too small: want %d\n",
+			       total_size);
+		abort();
+	}
 
 	total_size = 0;
 	for (i = 0; i < ARRAY_SIZE(runtime_features); i++)
 		total_size += strlen(runtime_features[i].name) + 2;
-	BUG_ON(BTRFS_FEATURE_STRING_BUF_SIZE < total_size);
+	if (BTRFS_FEATURE_STRING_BUF_SIZE < total_size) {
+		internal_error("string buffer for freature list too small: want %d\n",
+			       total_size);
+		abort();
+	}
 }
 
 static size_t get_feature_array_size(enum feature_source source)
@@ -335,6 +389,9 @@ static void parse_features_to_string(char *buf,
 	for (i = 0; i < array_size; i++) {
 		const struct btrfs_feature *feat = get_feature(i, source);
 
+		if (feature_name_is_alias(feat))
+			continue;
+
 		if (features->compat_ro_flags & feat->compat_ro_flag ||
 		    features->incompat_flags & feat->incompat_flag ||
 		    features->runtime_flags & feat->runtime_flag) {
@@ -412,7 +469,12 @@ static void list_all_features(const struct btrfs_mkfs_features *allowed,
 		      feat->runtime_flag & allowed->runtime_flags))
 			continue;
 
-		fprintf(stderr, "%-20s- %s (", feat->name, feat->desc);
+		fprintf(stderr, "%-20s- %s", feat->name, feat->desc);
+		if (feature_name_is_alias(feat)) {
+			fprintf(stderr, "\n");
+			continue;
+		}
+		fprintf(stderr, " (");
 		if (feat->compat_ver) {
 			fprintf(stderr, "compat=%s", feat->compat_str);
 			sep = ", ";
@@ -578,8 +640,10 @@ int btrfs_check_sectorsize(u32 sectorsize)
 
 	if (!sectorsize_checked)
 		warning(
-"the filesystem may not be mountable, sectorsize %u doesn't match page size %u",
-			sectorsize, page_size);
+"sectorsize %u does not match host CPU page size %u, with kernels 6.x and up\n"
+"\t the 4KiB sectorsize is supported on all architectures but other combinations\n"
+"\t may fail the filesystem mount, use \"--sectorsize %u\" to override that\n",
+			sectorsize, page_size, page_size);
 	return 0;
 }
 
@@ -619,6 +683,35 @@ int btrfs_check_features(const struct btrfs_mkfs_features *features,
 	return 0;
 }
 
+static bool tree_search_v2_supported = false;
+static bool tree_search_v2_initialized = false;
+
+/*
+ * Call the highest supported TREE_SEARCH ioctl version, autodetect support.
+ */
+int btrfs_tree_search_ioctl(int fd, struct btrfs_tree_search_args *sa)
+{
+	/* On first use check the supported status and save it. */
+	if (!tree_search_v2_initialized) {
+#if 0
+		/*
+		 * Keep using v1 until v2 is fully tested, in some cases it
+		 * does not return properly formatted results in the buffer.
+		 */
+		if (btrfs_tree_search2_ioctl_supported(fd) == 1)
+			tree_search_v2_supported = true;
+#endif
+		tree_search_v2_initialized = true;
+	}
+	sa->use_v2 = tree_search_v2_supported;
+
+	if (sa->use_v2) {
+		sa->args2.buf_size = BTRFS_TREE_SEARCH_V2_BUF_SIZE;
+		return ioctl(fd, BTRFS_IOC_TREE_SEARCH_V2, &sa->args2);
+	}
+	return ioctl(fd, BTRFS_IOC_TREE_SEARCH, &sa->args1);
+}
+
 /*
  * Check if the BTRFS_IOC_TREE_SEARCH_V2 ioctl is supported on a given
  * filesystem, opened at fd
@@ -639,10 +732,10 @@ int btrfs_tree_search2_ioctl_supported(int fd)
 	 */
 	sk->tree_id = BTRFS_ROOT_TREE_OBJECTID;
 	sk->min_objectid = BTRFS_EXTENT_TREE_OBJECTID;
-	sk->max_objectid = BTRFS_EXTENT_TREE_OBJECTID;
 	sk->min_type = BTRFS_ROOT_ITEM_KEY;
-	sk->max_type = BTRFS_ROOT_ITEM_KEY;
 	sk->min_offset = 0;
+	sk->max_objectid = BTRFS_EXTENT_TREE_OBJECTID;
+	sk->max_type = BTRFS_ROOT_ITEM_KEY;
 	sk->max_offset = (u64)-1;
 	sk->min_transid = 0;
 	sk->max_transid = (u64)-1;
