@@ -22,6 +22,8 @@
 #include "kerncompat.h"
 #include <stdbool.h>
 #include <stddef.h>
+#include <sys/stat.h>
+#include "kernel-lib/list.h"
 #include "kernel-shared/volumes.h"
 #include "common/fsfeatures.h"
 
@@ -39,6 +41,21 @@ enum exclusive_operation {
 	BTRFS_EXCLOP_UNKNOWN = -1,
 };
 
+static inline u32 btrfs_type_to_imode(u8 type)
+{
+	static u32 imode_by_btrfs_type[] = {
+		[BTRFS_FT_REG_FILE]	= S_IFREG,
+		[BTRFS_FT_DIR]		= S_IFDIR,
+		[BTRFS_FT_CHRDEV]	= S_IFCHR,
+		[BTRFS_FT_BLKDEV]	= S_IFBLK,
+		[BTRFS_FT_FIFO]		= S_IFIFO,
+		[BTRFS_FT_SOCK]		= S_IFSOCK,
+		[BTRFS_FT_SYMLINK]	= S_IFLNK,
+	};
+
+	return imode_by_btrfs_type[(type)];
+}
+
 /* 2 for "0x", 2 for each byte, plus nul */
 #define BTRFS_CSUM_STRING_LEN		(2 + 2 * BTRFS_CSUM_SIZE + 1)
 void btrfs_format_csum(u16 csum_type, const u8 *data, char *output);
@@ -86,12 +103,26 @@ struct btrfs_config {
 	 *   > 0: verbose level
 	 */
 	int verbose;
+	/* Command line request to skip any modification actions. */
+	int dry_run;
+	struct list_head params;
 };
 extern struct btrfs_config bconf;
 
+struct config_param {
+	struct list_head list;
+	const char *key;
+	const char *value;
+};
+
 void btrfs_config_init(void);
 void bconf_be_verbose(void);
 void bconf_be_quiet(void);
+void bconf_add_param(const char *key, const char *value);
+void bconf_save_param(const char *str);
+void bconf_set_dry_run(void);
+bool bconf_is_dry_run(void);
+const char *bconf_param_value(const char *key);
 
 /* Pseudo random number generator wrappers */
 int rand_int(void);
@@ -102,6 +133,8 @@ u64 rand_u64(void);
 unsigned int rand_range(unsigned int upper);
 void init_rand_seed(u64 seed);
 
+bool get_env_bool(const char *env_name);
+
 char *btrfs_test_for_multiple_profiles(int fd);
 int btrfs_warn_multiple_profiles(int fd);
 void btrfs_warn_experimental(const char *str);
