@@ -673,28 +673,28 @@ insert:
 	return root;
 }
 
-static int btrfs_global_roots_compare_keys(struct rb_node *node,
-					   void *data)
+static int btrfs_global_roots_compare_keys(const struct rb_node *node,
+					   const void *data)
 {
-	struct btrfs_key *key = (struct btrfs_key *)data;
-	struct btrfs_root *root;
+	const struct btrfs_key *key = (struct btrfs_key *)data;
+	const struct btrfs_root *root;
 
 	root = rb_entry(node, struct btrfs_root, rb_node);
 	return btrfs_comp_cpu_keys(key, &root->root_key);
 }
 
-static int btrfs_global_roots_compare(struct rb_node *node1,
-				      struct rb_node *node2)
+static int btrfs_global_roots_compare(const struct rb_node *node1,
+				      const struct rb_node *node2)
 {
-	struct btrfs_root *root = rb_entry(node2, struct btrfs_root, rb_node);
+	const struct btrfs_root *root = rb_entry(node2, struct btrfs_root, rb_node);
 	return btrfs_global_roots_compare_keys(node1, &root->root_key);
 }
 
-static int btrfs_fs_roots_compare_objectids(struct rb_node *node,
-					    void *data)
+static int btrfs_fs_roots_compare_objectids(const struct rb_node *node,
+					    const void *data)
 {
-	u64 objectid = *((u64 *)data);
-	struct btrfs_root *root;
+	u64 objectid = *((const u64 *)data);
+	const struct btrfs_root *root;
 
 	root = rb_entry(node, struct btrfs_root, rb_node);
 	if (objectid > root->objectid)
@@ -705,12 +705,12 @@ static int btrfs_fs_roots_compare_objectids(struct rb_node *node,
 		return 0;
 }
 
-int btrfs_fs_roots_compare_roots(struct rb_node *node1, struct rb_node *node2)
+int btrfs_fs_roots_compare_roots(const struct rb_node *node1, const struct rb_node *node2)
 {
-	struct btrfs_root *root;
+	const struct btrfs_root *root;
 
 	root = rb_entry(node2, struct btrfs_root, rb_node);
-	return btrfs_fs_roots_compare_objectids(node1, (void *)&root->objectid);
+	return btrfs_fs_roots_compare_objectids(node1, &root->objectid);
 }
 
 int btrfs_global_root_insert(struct btrfs_fs_info *fs_info,
@@ -2369,12 +2369,12 @@ int btrfs_delete_and_free_root(struct btrfs_trans_handle *trans,
 }
 
 struct btrfs_root *btrfs_create_tree(struct btrfs_trans_handle *trans,
-				     struct btrfs_fs_info *fs_info,
 				     struct btrfs_key *key)
 {
-	struct extent_buffer *leaf;
+	struct btrfs_fs_info *fs_info = trans->fs_info;
 	struct btrfs_root *tree_root = fs_info->tree_root;
 	struct btrfs_root *root;
+	struct extent_buffer *leaf;
 	int ret = 0;
 
 	root = kzalloc(sizeof(*root), GFP_KERNEL);
