@@ -34,6 +34,7 @@
 #include "common/device-utils.h"
 #include "common/extent-cache.h"
 #include "common/internal.h"
+#include "common/string-utils.h"
 #include "common/messages.h"
 #include "mkfs/common.h"
 
@@ -44,7 +45,9 @@
 /* Pseudo write pointer value for conventional zone */
 #define WP_CONVENTIONAL			((u64)-2)
 
-#define EMULATED_ZONE_SIZE		SZ_256M
+#define DEFAULT_EMULATED_ZONE_SIZE		SZ_256M
+
+static u64 emulated_zone_size = DEFAULT_EMULATED_ZONE_SIZE;
 
 /*
  * Minimum / maximum supported zone size. Currently, SMR disks have a zone size
@@ -93,8 +96,22 @@ u64 zone_size(const char *file)
 	int ret;
 
 	/* Zoned emulation on regular device */
-	if (zoned_model(file) == ZONED_NONE)
-		return EMULATED_ZONE_SIZE;
+	if (zoned_model(file) == ZONED_NONE) {
+		const char *tmp;
+		u64 size = DEFAULT_EMULATED_ZONE_SIZE;
+
+		tmp = bconf_param_value("zone-size");
+		if (tmp) {
+			size = arg_strtou64_with_suffix(tmp);
+			if (!is_power_of_2(size) || size < BTRFS_MIN_ZONE_SIZE ||
+			    size > BTRFS_MAX_ZONE_SIZE) {
+				error("invalid emulated zone size %llu", size);
+				exit(1);
+			}
+		}
+		emulated_zone_size = size;
+		return emulated_zone_size;
+	}
 
 	ret = device_get_queue_param(file, "chunk_sectors", chunk, sizeof(chunk));
 	if (ret <= 0)
@@ -125,7 +142,7 @@ static u64 max_zone_append_size(const char *file)
 static int emulate_report_zones(const char *file, int fd, u64 pos,
 				struct blk_zone *zones, unsigned int nr_zones)
 {
-	const sector_t zone_sectors = EMULATED_ZONE_SIZE >> SECTOR_SHIFT;
+	const sector_t zone_sectors = emulated_zone_size >> SECTOR_SHIFT;
 	struct stat st;
 	sector_t bdev_size;
 	unsigned int i;
@@ -323,7 +340,7 @@ static int report_zones(int fd, const char *file,
 	/* Allocate a zone report */
 	rep_size = sizeof(struct blk_zone_report) +
 		   sizeof(struct blk_zone) * BTRFS_REPORT_NR_ZONES;
-	rep = malloc(rep_size);
+	rep = kmalloc(rep_size, GFP_KERNEL);
 	if (!rep) {
 		error_msg(ERROR_MSG_MEMORY, "zone report");
 		exit(1);
@@ -369,7 +386,7 @@ static int report_zones(int fd, const char *file,
 			 zone[rep->nr_zones - 1].len;
 	}
 
-	free(rep);
+	kfree(rep);
 
 	return 0;
 }
@@ -378,16 +395,23 @@ static int report_zones(int fd, const char *file,
  * Discard blocks in the zones of a zoned block device. Process this with zone
  * size granularity so that blocks in conventional zones are discarded using
  * discard_range and blocks in sequential zones are reset though a zone reset.
+ *
+ * We need to ensure that zones outside of the fs are not active, so that the fs
+ * can use all the active zones. Return EBUSY if there is an active zone.
  */
-int btrfs_reset_all_zones(int fd, struct btrfs_zoned_device_info *zinfo)
+int btrfs_reset_zones(int fd, struct btrfs_zoned_device_info *zinfo, u64 byte_count)
 {
 	unsigned int i;
 	int ret = 0;
 
 	ASSERT(zinfo);
+	ASSERT(IS_ALIGNED(byte_count, zinfo->zone_size));
 
 	/* Zone size granularity */
 	for (i = 0; i < zinfo->nr_zones; i++) {
+		if (byte_count == 0)
+			break;
+
 		if (zinfo->zones[i].type == BLK_ZONE_TYPE_CONVENTIONAL) {
 			ret = device_discard_blocks(fd,
 					     zinfo->zones[i].start << SECTOR_SHIFT,
@@ -402,7 +426,20 @@ int btrfs_reset_all_zones(int fd, struct btrfs_zoned_device_info *zinfo)
 
 		if (ret)
 			return ret;
+
+		byte_count -= zinfo->zone_size;
 	}
+	for (; i < zinfo->nr_zones; i++) {
+		const enum blk_zone_cond cond = zinfo->zones[i].cond;
+
+		if (zinfo->zones[i].type == BLK_ZONE_TYPE_CONVENTIONAL)
+			continue;
+		if (cond == BLK_ZONE_COND_IMP_OPEN ||
+		    cond == BLK_ZONE_COND_EXP_OPEN ||
+		    cond == BLK_ZONE_COND_CLOSED)
+			return EBUSY;
+	}
+
 	return fsync(fd);
 }
 
@@ -575,14 +612,14 @@ size_t btrfs_sb_io(int fd, void *buf, off_t offset, int rw)
 			return (rw == WRITE ? count : 0);
 		error("zoned: failed to read zone info of %u and %u: %m",
 		      zone_num, zone_num + 1);
-		free(rep);
+		kfree(rep);
 		return 0;
 	}
 
 	zones = (struct blk_zone *)(rep + 1);
 
 	ret = sb_log_location(fd, zones, rw, &mapped);
-	free(rep);
+	kfree(rep);
 	/*
 	 * Special case: no superblock found in the zones. This case happens
 	 * when initializing a file-system.
@@ -921,7 +958,7 @@ out:
 	if (!ret)
 		cache->write_offset = cache->alloc_offset;
 
-	free(alloc_offsets);
+	kfree(alloc_offsets);
 	return ret;
 }
 
