@@ -37,6 +37,8 @@ PUBLIC enum btrfs_util_error btrfs_util_sync(const char *path)
 	SAVE_ERRNO_AND_CLOSE(fd);
 	return err;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_sync(const char *path)
+LIBBTRFSUTIL_ALIAS(btrfs_util_sync);
 
 PUBLIC enum btrfs_util_error btrfs_util_sync_fd(int fd)
 {
@@ -48,6 +50,8 @@ PUBLIC enum btrfs_util_error btrfs_util_sync_fd(int fd)
 
 	return BTRFS_UTIL_OK;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_sync_fd(int fd)
+LIBBTRFSUTIL_ALIAS(btrfs_util_sync_fd);
 
 PUBLIC enum btrfs_util_error btrfs_util_start_sync(const char *path,
 						   uint64_t *transid)
@@ -63,6 +67,8 @@ PUBLIC enum btrfs_util_error btrfs_util_start_sync(const char *path,
 	SAVE_ERRNO_AND_CLOSE(fd);
 	return err;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_start_sync(const char *path, uint64_t *transid)
+LIBBTRFSUTIL_ALIAS(btrfs_util_start_sync);
 
 PUBLIC enum btrfs_util_error btrfs_util_start_sync_fd(int fd, uint64_t *transid)
 {
@@ -74,6 +80,8 @@ PUBLIC enum btrfs_util_error btrfs_util_start_sync_fd(int fd, uint64_t *transid)
 
 	return BTRFS_UTIL_OK;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_start_sync_fd(int fd, uint64_t *transid)
+LIBBTRFSUTIL_ALIAS(btrfs_util_start_sync_fd);
 
 PUBLIC enum btrfs_util_error btrfs_util_wait_sync(const char *path,
 						  uint64_t transid)
@@ -89,6 +97,8 @@ PUBLIC enum btrfs_util_error btrfs_util_wait_sync(const char *path,
 	SAVE_ERRNO_AND_CLOSE(fd);
 	return err;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_wait_sync(const char *path, uint64_t transid)
+LIBBTRFSUTIL_ALIAS(btrfs_util_wait_sync);
 
 PUBLIC enum btrfs_util_error btrfs_util_wait_sync_fd(int fd, uint64_t transid)
 {
@@ -100,3 +110,5 @@ PUBLIC enum btrfs_util_error btrfs_util_wait_sync_fd(int fd, uint64_t transid)
 
 	return BTRFS_UTIL_OK;
 }
+PUBLIC enum btrfs_util_error btrfs_util_fs_wait_sync_fd(int fd, uint64_t transid)
+LIBBTRFSUTIL_ALIAS(btrfs_util_wait_sync_fd);
