@@ -21,17 +21,23 @@
 #ifndef __BTRFS_MKFS_ROOTDIR_H__
 #define __BTRFS_MKFS_ROOTDIR_H__
 
-#include "kernel-lib/list.h"
+#include "kerncompat.h"
+#include <sys/types.h>
+#include <stdbool.h>
 
-struct directory_name_entry {
-	const char *dir_name;
-	char *path;
-	ino_t inum;
+struct btrfs_fs_info;
+struct btrfs_root;
+
+struct rootdir_subvol {
 	struct list_head list;
+	char dir[PATH_MAX];
+	char full_path[PATH_MAX];
+	bool is_default;
+	bool readonly;
 };
 
-int btrfs_mkfs_fill_dir(const char *source_dir, struct btrfs_root *root,
-			bool verbose);
+int btrfs_mkfs_fill_dir(struct btrfs_trans_handle *trans, const char *source_dir,
+			struct btrfs_root *root, struct list_head *subvols);
 u64 btrfs_mkfs_size_dir(const char *dir_name, u32 sectorsize, u64 min_dev_size,
 			u64 meta_profile, u64 data_profile);
 int btrfs_mkfs_shrink_fs(struct btrfs_fs_info *fs_info, u64 *new_size_ret,
