@@ -1,15 +1,16 @@
 #!/bin/bash
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
+
+check_prereq btrfs
 
 setup_root_helper
-check_prereq btrfs
 
 # redefine the one provided by common
 check_image() {
 	local image
 
-	image=$1
+	image="$1"
 	run_check cp "$image" "$image".scratch
 	run_mayfail "$TOP/btrfs" rescue chunk-recover -y -v "$image".scratch
 	rm -- "$image".scratch
