@@ -1,13 +1,12 @@
 #!/bin/bash
-# Basic check if mkfs supports the runtime feature free-space-tree
+# Check that free-space-tree does not create space bitmaps on an empty filesystem
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
 
 setup_root_helper
-
 setup_loopdevs 4
 prepare_loopdevs
 dev1=${loopdevs[1]}
