@@ -113,8 +113,12 @@ static const char * const tune_usage[] = {
 	"UUID changes:",
 	OPTLINE("-u", "rewrite fsid, use a random one"),
 	OPTLINE("-U UUID", "rewrite fsid to UUID"),
-	OPTLINE("-m", "change fsid in metadata_uuid to a random UUID incompat change, more lightweight than -u|-U)"),
-	OPTLINE("-M UUID", "change fsid in metadata_uuid to UUID"),
+	OPTLINE("-m", "change fsid to a random UUID, copy original fsid into "
+		      "metadata_uuid if it's not NULL, this is an incompat change "
+		      "(more lightweight than -u|-U)"),
+	OPTLINE("-M UUID", "change fsid to UUID, copy original fsid into "
+		      "metadata_uuid if it's not NULL, this is an incompat change "
+		      "(more lightweight than -u|-U)"),
 	"",
 	"General:",
 	OPTLINE("-f", "allow dangerous operations, make sure that you are aware of the dangers"),
@@ -162,7 +166,7 @@ enum btrfstune_group_enum {
 
 static bool btrfstune_cmd_groups[BTRFSTUNE_NR_GROUPS] = { 0 };
 
-static unsigned int btrfstune_count_set_groups()
+static unsigned int btrfstune_count_set_groups(void)
 {
 	int ret = 0;
 
@@ -181,7 +185,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 {
 	struct btrfs_root *root;
 	struct btrfs_fs_info *fs_info;
-	unsigned ctree_flags = OPEN_CTREE_WRITES;
+	unsigned ctree_flags = OPEN_CTREE_WRITES | OPEN_CTREE_EXCLUSIVE;
 	int seeding_flag = 0;
 	u64 seeding_value = 0;
 	int random_fsid = 0;
